<?php
/**
* @version 00.04.03
* @package polaring
* @license This component in released under the GNU/GPL License
*
* File: functions/fileFunctions.php
* Role: General functions relating to files 
**/

class fileFunctions {
    
    function fileToString($file) {
	/********************\
	 * Input: $file
	 * Output: Variable, file read in to string
	\********************/
	$data = "";
	$fOpen = fopen("$file", "rb");
	if ( ! $fOpen ) {
	    die("<p>Couldn't open input</p>");
	} else {
	    $line = fread($fOpen, 8192);
	    do {
		$data .= $line;
	    } while ( $line = fread($fOpen, 8192) );
	}
	return $data;
    } // end fileToString


    function readWorkoutDir($dir, $fTypes) {
	$files = array();
	$validFTypes = explode(", ", $fTypes);
	$stack[] = $dir;
	if ( is_dir($_SESSION['uploadDir']) ) {
	    $stack[] = $_SESSION['uploadDir'];
	}

	while ( $stack ) {
	    $current_dir = array_pop($stack);
	    if ( $odir = opendir($current_dir) ) {
		while ( ($file = readdir($odir)) !== false ) {
		    if ( $file !== "." && $file !== ".." ) {
			$current_file = $current_dir."/".$file;
			if ( is_file($current_file) ) {
			    $fExtension = substr(strtolower($file), strlen($file) - 3);
			    if ( in_array($fExtension, $validFTypes) ) {
				$files[] = $current_file;
			    }
			} elseif ( is_dir($current_file) ) {
			    $stack[] = $current_file;
			}
		    }
		}
	    }
	}

	$workoutsWDate = fileFunctions::getWorkoutDates($files);

	return($workoutsWDate);
    } // end function readWorkoutDir


    function getWorkoutDates($files) {
	$wWdate = array();
	$filesMd5 = array();

	// A md5sum-based check for files in databases.
	// Useful for checking changes in (the MySQL) workout-cache,
	// however, it does make things a bit slower, espescially if
	// there's a lot of files. It's most useful if there's only
	// one large file (thinking of HST:s here). 
	// Maybe there should be a option to disable it?
	foreach ( $files as $file ) {
	    $filesMd5[$file] = md5_file($file);
	}

	if ( $_SESSION['mysql']['on'] ) {
	    $sqlMd5 = databas::fetchMd5();

	    for ( $i = 0; $i < count($sqlMd5['md5']); $i++ ) {
		$key = array_search($sqlMd5['md5'][$i], $filesMd5);
		if ( $key === false ) {
		    // File found in SQL not in file-array!
		    // Remove all startdates from SQL that the file contained.
		    databas::deleteDateStartTime($sqlMd5['dateStartTime'][$i]);
		}
	    }
	}

	foreach ( $files as $file ) {
	    $fInfo = pathinfo($file);

	    if ( $_SESSION['mysql']['on'] ) {
		// Fetch all workouts from MySQL that it has stored by file
		$workoutsSQL = databas::fetchWOfiles($file, "");
	    } else {
		$workoutsSQL = false;
	    }

	    if ( !$workoutsSQL ) {
		// We're not using MySQL, or it wasn't found.
		// Either way, we have to parse the file.
		$fileInMySQL = false;
	    } else {
		// Workouts found in MySQL.
		// Reading them instead of parsing file.
		$fileInMySQL = true;
		unset($w);
		foreach  ( $workoutsSQL as $w ) {
		    if ( $w['duplicate'] == "0" ) {
			$key = date("YmdHi", $w['dateStartTime']);
			$wWDate[$key] = $w;
		    }
		}
	    }

	    if ( !$fileInMySQL ) {
		// No MySQL data for file, parsing it.
		unset($workout);
		unset($key);

		// If we've got a SRD or HRM file:
		if ( $fInfo['extension'] == "srd" || $fInfo['extension'] == "hrm" ) {
		    if ( $fInfo['extension'] == "srd" ) {
			$workout = getSrdForMonthTableView($file);
		    } elseif ( $fInfo['extension'] == "hrm" ) {
			$workout = getHrmComplete($file);
		    }

		    $workout['file'] = $file;
		    $workout['md5sum'] = $filesMd5[$file];
		    $key = date("YmdHi", $workout['dateStartTime']);
		    $wWDate[$key] = $workout;
		    if ( $_SESSION['mysql']['on'] ) {
			databas::storeWorkout($workout);
		    }
		} elseif ( $fInfo['extension'] == "hst" ) {
		    // We have a HST-file.
		    if ( $xmlArray = garmin::fileToArray($file) ) {
			if ( $workoutsHst = garmin::getWorkouts($xmlArray) ) {
			    if ( $workouts = garmin::parseHst($workoutsHst) ) {
				if ( is_array($workouts) ) {
				    foreach ( $workouts as $workout ) {
					if ( $workout['dateStartTime'] ) {
					    if ( !$workout['file'] ) {
						$workout['file'] = $file;
						$workout['md5sum'] = $filesMd5[$file];
					    }
					    $key = date("YmdHi", $workout['dateStartTime']);
					    $wWDate[$key] = $workout;
					    if ( $_SESSION['mysql']['on'] ) {
						$SQLFile = databas::fetchWOfiles("", $workout['dateStartTime']);
						if ( count($SQLFile) == 0 || !$SQLFile ) {
						    databas::storeWorkout($workout);
						    databas::storeWSamples($workout);
						} else {
						    databas::storeDuplicate($workout);
						}
					    }
					}
				    }
				}  else {
				    //echo "<p>There's no workouts!</p>";
				}
			    } else {
				//echo "<p>Can't parse hst!</p>";
			    }
			} else {
			    //echo "<p>Something wrong with workouts!</p>";
			}
		    } else {
			//echo "<p>NO FILE</p>";
		    }
		}
	    }
	}

	if ( count($wWDate) != "0" ) {
	    ksort($wWDate);
	}
	return($wWDate);
    } // end function getWorkoutDates


    function selectFullWorkout($dateTime) {
	$file = $_SESSION['workouts'][$dateTime]['file'];
	$fileInfo = pathinfo($file);
	if ( $fileInfo['extension'] == "hrm" ) {
	    if ( !$_SESSION['workouts'][$dateTime]['sampleData'] ) {
		$sampleData = getHrmComplete($_SESSION['workouts'][$dateTime]['file']);
		calculateTimeSamples($sampleData);
		calculateDistanceSamples($sampleData);
		trimAltitude($sampleData);
		$_SESSION['workouts'][$dateTime]['sampleData'] = $sampleData;
	    }
	    $selected = $_SESSION['workouts'][$dateTime]['sampleData'];
	} elseif ( $fileInfo['extension'] == "srd" ) {
	    if ( !$_SESSION['workouts'][$dateTime]['hrmData'] ) {
		$_SESSION['workouts'][$dateTime]['hrmData'] = srdTohrm($fileInfo);
	    }
	    $selected = $_SESSION['workouts'][$dateTime]['hrmData'];
	    calculateTimeSamples($selected);
	    calculateDistanceSamples($selected);
	    trimAltitude($selected);
	} elseif ( $fileInfo['extension'] == "hst" ) {
	    $selected = $_SESSION['workouts'][$dateTime];
	    // fetch samples
	    if ( !$selected['sampleTime'] ) {
		$wSamples = databas::fetchWSamples($file, $_SESSION['workouts'][$dateTime]['dateStartTime']);
		$selected['sampleHr'] = $wSamples['sampleHr'];
		$selected['sampleSpeed'] = $wSamples['sampleSpeed'];
		$selected['sampleTime'] = $wSamples['sampleTime'];
		$selected['sampleAltitude'] = $wSamples['sampleAltitude'];
		$selected['sampleDistance'] = $wSamples['sampleDistance'];
		$selected['sampleLocationLat'] = $wSamples['sampleLocationLat'];
		$selected['sampleLocationLong'] = $wSamples['sampleLocationLong'];
	    }
	    unset($_SESSION['workouts'][$dateTime]);
	    $_SESSION['workouts'][$dateTime] = $selected;
	}


	return($selected);
    } // end selectFullWorkout


} // end class fileFunctions

?>
