<?php
/**
* @version 00.04.03
* @package polaring
* @license This component in released under the GNU/GPL License
*
* File: functions/garminParser.php
* Role: Functions for parsing garmin exercise-files
**/ 

class garmin {

    function getXmlRuns($string) {
	$runs = array();

	$blocksWEnd = split("<Run>", $string);
	foreach ( $blocksWEnd as $blocks ) {
	    if ( strstr($blocks, "<Training") ) {
		$endPos = strpos($blocks, "<Training");
	    } elseif ( strstr($blocks, "</Run>") ) {
		$endPos = strpos($blocks, "</Run>");
	    }
	    $runs[] = trim(substr($blocks, 0, $endPos));
	}
	return($runs);
    } // end function getXmlRuns


    function stripTraining($string) {
	$endPos = strpos($string, "<Training");

	if ( $endPos ) {
	    $returnString = substr($string, 0, $endPos);
	} else {
	    return($string);
	}
	return($returnString);
    } // end function stripTraining


    function fileToArray($file) {
	// Read XML-file into string
	$string = fileFunctions::fileToString($file);
	// Split (xml-)string into blocks
	$runsString = garmin::getXmlRuns($string);
	/*echo "<pre>\n";
	print_r($runsString);
	echo "</pre>\n";
	die(); */

	$xml = array();

	// Read each block into separate arrays
	foreach ( $runsString as $runString ) {
	    if ( $runString ) {
		//$justLaps = garmin::stripTraining($runString);
		$justLaps = $runString;
		$run = "\n  <Run attrib=\"IgnoreMe\">\n".$justLaps."\n  </Run>\n";
		$runXML = GetXMLTree($run);
		$xml[] = $runXML['Run'];
	    } else {
		return(false);
	    }
	}

	return($xml);
    } // end fileToArray


    function timeHst2Unix($timeDate) {
	$unixTime = strtotime($timeDate);
	return($unixTime);
    } // end function timeHst2Unix


    function trackData($track, $tpInfo) {
	$nTp = $tpInfo['nTp'];
	$trackpoints = $tpInfo['trackpoints'];

	$possibleTrackpointEntries = array(
	    "Time", 
	    "DistanceMeters", 
	    "AltitudeMeters",
	    "SensorState",
	    "HeartRateBpm");

	if ( !is_array($track) ) {
	    break;
	}
	//print_r(array_keys($track));
	foreach ( array_keys($track) as $trackKey ) {
	    if ( $trackKey == "Trackpoint" ) {
		//echo "GOT TRACKPOINT!\n";
		//print_r(array_keys($track[$trackKey]));
		foreach ( $track[$trackKey] as $mTrackpoint ) {
		    if ( !array_search($mTrackpoint['Time'], $trackpoints) &&
			strtotime($mTrackpoint['Time']) ) {
			    foreach ( $possibleTrackpointEntries as $entry ) {
				$trackpoints[$nTp][$entry] = $mTrackpoint[$entry];
			    }
			    if ( is_array($mTrackpoint['Position']) ) {
				$trackpoints[$nTp]['Position'] = array(
				    "LatitudeDegrees" => $mTrackpoint['Position']['LatitudeDegrees'],
				    "LongitudeDegrees" => $mTrackpoint['Position']['LongitudeDegrees']);
			    }
			    $nTp++;
			}
		} 
	    } elseif ( $trackKey == "Time" ) {
		//echo "GOT TIME! Always only one trackpoint?";
		$trackpoints[$nTp]['Time'] = $track[$trackKey];
		$nTp++;
	    } else {
		//print_r($track[$trackKey]);
		//print_r(array_keys($track[$trackKey]));
		if ( !array_search($track[$trackKey]['Time'], $trackpoints) &&
		    strtotime($track[$trackKey]['Time']) ) {
			foreach ( $possibleTrackpointEntries as $entry ) {
			    if ( $track[$trackKey][$entry] ) {
				$trackpoints[$nTp][$entry] = $track[$trackKey][$entry];
			    }
			}
			if ( is_array($track[$trackKey]['Position']) ) {
			    $trackpoints[$nTp]['Position'] = array(
				"LatitudeDegrees" => $track[$trackKey]['Position']['LatitudeDegrees'],
				"LongitudeDegrees" => $track[$trackKey]['Position']['LongitudeDegrees']);
			}
			$nTp++;
		    }
	    }
	}   
	$info['nTp'] = $nTp;
	$info['trackpoints'] = $trackpoints;
	//print_r($trackpoints);

	return($info);
    } // end function trackData


            
    function getWorkouts($xml) {
	$i = 0;
	$possibleTrackpointEntries = array(
	    "Time", 
	    "DistanceMeters", 
	    "AltitudeMeters",
	    "SensorState",
	    "HeartRateBpm");

	if ( !is_array($xml) ) {
	    return(false);
	}

	echo "<pre>";
	//print_r(array_keys($xml));
	foreach ( $xml as $laps ) {
	    if ( is_array($laps) ) {
		//print_r(array_keys($laps));
		array_shift($laps);
		foreach ( $laps as $lap ) {
		    //print_r(array_keys($lap));
		    $trackpoints = array();
		    $nTp = 0;
		    $nLaps = 0;
		    $nHrAvg = 0;

		    if ( is_array($lap['0']) ) {
			// There's several laps.
			//echo "ARRAY\n";
			$TotTimSec = 0;
			$DistMeter = 0;
			$Calories = 0;
			$AvgHrate = 0;
			$MaxHrate = 0;
			$j = 0;

			foreach ( $lap as $mlap ) {
			    if ( $mlap['TotalTimeSeconds'] ) {
				$nLaps++;
				//print_r(array_keys($mlap));
				$TotTimSec += $mlap['TotalTimeSeconds'];
				$DistMeter += $mlap['DistanceMeters'];
				$Calories += $mlap['Calories'];
				if ( $mlap['AverageHeartRateBpm'] ) {
				    $AvgHrate += $mlap['AverageHeartRateBpm'];
				    $nHrAvg++;
				    if ( $mlap['MaximumHeartRateBpm'] > $MaxHrate ) {
					$MaxHrate = $mlap['MaximumHeartRateBpm'];
				    }
				}
				if ( !isset($StTime) && $lap['StartTime'] ) {
				    $StTime = $lap['StartTime'];
				}

				//print_r(($mlap['Track']));    
				if ( $mlap['Track'] ) {
				    foreach ( $mlap['Track'] as $track ) {
					$tpInfo['nTp'] = $nTp;
					$tpInfo['trackpoints'] = $trackpoints;
					$recTp = garmin::trackData($track, $tpInfo);
					$trackpoints = $recTp['trackpoints'];
					$nTp = $recTp['nTp'];
				    }
				}
			    }
			}
			//print_r($trackpoints);
			$thelap['TotalTimeSeconds'] = $TotTimSec;
			$thelap['DistanceMeters'] = $DistMeter;
			if ( $nHrAvg != 0 ) {
			    $thelap['AverageHeartRateBpm'] = $AvgHrate / $nHrAvg;
			    $thelap['MaximumHeartRateBpm'] = $MaxHrate;
			}
			$thelap['StartTime'] = $StTime;
			$thelap['Calories'] = $Calories;
			$thelap['Track'] = $trackpoints;

			unset($nTp, $trackpoints, $tpInfo, $recTp, $mlap);
		    } else {
			// There's only one lap.
			//echo "NOTANARRAY \n";
			$nLaps++;
			//print_r($lap);
			$thelap = array(
			    "TotalTimeSeconds" => $lap['TotalTimeSeconds'],
			    "DistanceMeters" => $lap['DistanceMeters'],
			    "Calories" => $lap['Calories'],
			    "Intensity" => $lap['Intensity'],
			    "Cadence" => $lap['Cadence']);
			if ( $lap['AverageHeartRateBpm'] ) {
			    $nHrAvg++;
			    $thelap['AverageHeartRateBpm'] = $lap['AverageHeartRateBpm'];
			    $thelap['MaximumHeartRateBpm'] = $lap['MaximumHeartRateBpm'];
			}
			//print_r(array_keys($lap['Track']));
			if ( $lap['Track'] ) {
			    foreach ( $lap['Track'] as $track ) {
				$tpInfo['nTp'] = $nTp;
				$tpInfo['trackpoints'] = $trackpoints;
				$recTp = garmin::trackData($track, $tpInfo);
				$trackpoints = $recTp['trackpoints'];
				$nTp = $recTp['nTp'];
			    }
			}
			$thelap['Track'] = $trackpoints;
			unset($lap, $trackpoints);
		    }
		    //print_r($thelap);
		    $workout[$i] = array();
		    $workout[$i]['MaximumSpeed'] = 0;
		    $workout[$i]['dateStartTime'] = 0;
		    $workout[$i]['numberOfLaps'] = $nLaps;
		    $workout[$i]['ascent'] = 0;
		    $workout[$i]['descent'] = 0;
		    $workout[$i]['sampleAltitude'] = array();
		    $workout[$i]['sampleSpeed'] = array();
		    $workout[$i]['sampleHr'] = array();
		    $workout[$i]['sampleTime'] = array();
		    $workout[$i]['sampleLocationLat'] = array();
		    $workout[$i]['sampleLocationLong'] = array();

		    $nAlt = 0;
		    $ascent = 0;
		    $descent = 0;

		    if ( is_array($thelap['Track']) ) {
			//print_r(array_keys($thelap['Track']));
			$workout[$i]['TotalTimeSeconds'] = $thelap['TotalTimeSeconds'];
			$workout[$i]['DistanceMeters'] = $thelap['DistanceMeters'];
			$workout[$i]['Calories'] = $thelap['Calories'];
			if ( $thelap['AverageHeartRateBpm'] ) {
			    $workout[$i]['AverageHeartRateBpm'] = $thelap['AverageHeartRateBpm'];
			    $workout[$i]['MaximumHeartRateBpm'] = $thelap['MaximumHeartRateBpm'];
			}
			if ( $thelap['StartTime'] ) {
			    $lapUnixTime = garmin::timeHst2Unix($thelap['StartTime']);
			    if ( $workout[$i]['dateStartTime'] == "0" ) {
				$workout[$i]['dateStartTime'] = $lapUnixTime;
			    } elseif ( $lapUnixTime < $workout[$i]['dateStartTime'] ) {
				$workout[$i]['dateStartTime'] = $lapUnixTime;
			    }
			} else {
			    $workout[$i]['dateStartTime'] = false;
			}

			//echo "TRACKPOINTS: $nTp \n";
			//echo "TRACKPOINTS, TOTAL: ".count($trackpoints)."\n";

			$n = 0;

			foreach ( $thelap['Track'] as $trackpoint ) {
			    //print_r($trackpoint);
			    //echo "TIME: ".$trackpoint['Time']." \n";
			    $time = garmin::timeHst2Unix($trackpoint['Time']);
			    if ( !$workout[$i]['dateStartTime'] ) {
				$workout[$i]['dateStartTime'] = $time;
			    }
			    if ( !isset($startTime) ) {
				$startTime = $time;
				$workout[$i]['sampleTime'][$n] = "00:00:00";
			    } else {
				$diffTime = $time - $startTime;
				$workout[$i]['sampleTime'][$n] = fromSecondsToFull($diffTime);
			    }
			    $distMeters = $trackpoint['DistanceMeters'];
			    if ( !isset($lastMeters) ) {
				$lastMeters = 0;
				$lastDiffTime = 0;
				$workout[$i]['sampleSpeed'][$n] = 0;
			    } else {
				$travMeters = $distMeters - $lastMeters;
				$travTime = $diffTime - $lastDiffTime;
				$lastMeters = $trackpoint['DistanceMeters'];
				$lastDiffTime = $lastDiffTime + $travTime;
				if ( $travTime > 0 ) {
				    $kmh = (3600 / $travTime) * (round($travMeters / 1000, 3));
				    // subzero speed? I don't think so.
				    if ( $kmh < 0 ) {
					$kmh = 0;
				    }
				    // Speed-check 
				    if ( isset($prevKmh) ) {
					$limitUp = 1.5;
					$limitLo = 0.5;
					if ( $kmh > $limitUp * $prevKmh ) {
					    $kmh = $limitUp * $prevKmh;
					} elseif ( $kmh < $limitLo * $prevKmh ) {
					    $kmh = $limitLo * $prevKmh;
					}
					$prevKmh = $kmh;
				    } else {
					$prevKmh = $kmh;
				    }
				} else {
				    $kmh = "";
				}
				$workout[$i]['sampleSpeed'][$n] = $kmh;
			    }
			    if ( $kmh > $workout[$i]['MaximumSpeed'] ) {
				$workout[$i]['MaximumSpeed'] = $kmh;
			    }
			    if ( is_array($trackpoint['Position']) ) {
				$workout[$i]['sampleLocationLat'][$n] = $trackpoint['Position']['LatitudeDegrees'];
				$workout[$i]['sampleLocationLong'][$n] = $trackpoint['Position']['LongitudeDegrees'];
			    } else {
				$workout[$i]['sampleLocationLat'][$n] = "";
				$workout[$i]['sampleLocationLong'][$n] = "";
			    }
			    $workout[$i]['sampleDistance'][$n] = round(
				$trackpoint['DistanceMeters'] / 1000, 3);
			    if ( $trackpoint['HeartRateBpm'] ) {
				$workout[$i]['sampleHr'][$n] = $trackpoint['HeartRateBpm'];
			    }
			    if ( $trackpoint['AltitudeMeters'] ) {
				$workout[$i]['sampleAltitude'][$n] = $trackpoint['AltitudeMeters'];
				if ( !isset($maxAlt) ) {
				    $maxAlt = $trackpoint['AltitudeMeters'];
				} elseif ( $trackpoint['AltitudeMeters'] > $maxAlt ) {
				    $maxAlt = $trackpoint['AltitudeMeters'];
				}
				if ( !isset($minAlt) ) {
				    $minAlt = $trackpoint['AltitudeMeters'];
				} elseif ( $trackpoint['AltitudeMeters'] < $minAlt ) {
				    $minAlt = $trackpoint['AltitudeMeters'];
				}
				if ( !isset($lastAlt) ) {
				    $lastAlt = $trackpoint['AltitudeMeters'];
				} else {
				    $altDiff = $lastAlt - $trackpoint['AltitudeMeters'];
				    if ( $altDiff > 0 ) {
					$ascent += $altDiff;
				    } elseif ( $altDiff < 0 ) {
					$descent += -1 * $altDiff;
				    }
				    $lastAlt = $trackpoint['AltitudeMeters'];
				}
				$nAlt++;
			    }
			    $n++;
			}
		    }
		    unset($thelap, $lastAlt);
		}

		// Number of samples
		$workout[$i]['numberOfSamples'] = count($workout[$i]['sampleTime']);

		// Altitude
		if ( $nAlt != 0 ) {
		    $workout[$i]['ascent'] = $ascent;
		    $workout[$i]['descent'] = $descent;
		    $workout[$i]['altitudeMin'] = $minAlt;
		    $workout[$i]['altitudeMax'] = $maxAlt;
		    $workout[$i]['altitudeAvg'] = array_sum(
			$workout[$i]['sampleAltitude']) / $nAlt;
		}


		// Calculate average speed
		$workout[$i]['AverageSpeed'] = round((3600 / $workout[$i]['TotalTimeSeconds']) * (round($workout[$i]['DistanceMeters'] / 1000, 3)), 2);

		// If we don't have an average (or maximum) heartrate,
		// but do got samples: calculate average (and, possibly,
		// maximum).
		$sumHrAvg = array_sum($workout[$i]['sampleHr']);
		if ( !$workout[$i]['AverageHeartRateBpm'] && 
		    $sumHrAvg > 0 ) {
			//echo "HAVEN'T GOT AVERAGEHEARTRATE, BUT GOT SAMPLEHR: CALCULATING AVGHR";
			$workout[$i]['AverageHeartRateBpm'] = round($sumHrAvg / count($workout[$i]['sampleHr']), 0);
		    }
		if ( !$workout[$i]['MaximumHeartRateBpm'] &&
		    $sumHrAvg > 0 ) {
			//echo "HAVEN'T GOT MAXIMUMHEARTRATE, BUT GOT SAMPLEHR: CALCULATING MAXHR";
			$workout[$i]['MaximumHeartRateBpm'] = max($workout[$i]['sampleHr']);
		    }
	    $i++;
	    unset($nTp, $trackpoints, $startTime, $lastMeters);
	    }
	    //echo "DATESTARTTIME: ".$workout[$i]['dateStartTime']."\n";
	    //echo "NUMOFSAMPLES : ".$workout[$i]['numberOfSamples']."\n";
	    //print_r($workout[$i]);
	}

	if ( $i == "0" || count($workout) == 0 ) {
	    return(false);
	}

	echo "</pre>";

	return($workout);

    } // end function getWorkouts


    function parseHst($workoutsHst) {
	if ( !is_array($workoutsHst) ) {
	    echo "<p>No workouts returned!</p>";
	    return(false);
	}

	foreach ( $workoutsHst as $w ) {
	    if ( !$w['dateStartTime'] ) {
		echo "<p>Workout has no dateStartTime!</p>";
		return(false);
	    }
	    $run = $run0['Lap']['0'];
	    $workout['monitor'] = "Garmin TrainingCenterDatabase";
	    $workout['dateStartTime'] = $w['dateStartTime'];
	    $workout['duration'] = fromSecondsToFull($w['TotalTimeSeconds']);
	    $workout['hrAvg'] = $w['AverageHeartRateBpm'];
	    $workout['hrMax'] = $w['MaximumHeartRateBpm'];
	    $workout['units'] = 0;
	    $workout['unitHr'] = "bpm";
	    $workout['unitSpeed'] = "km/h";
	    $workout['unitCadence'] = "rpm";
	    $workout['unitAltitude'] = "m";
	    $workout['unitDistance'] = "km";
	    $workout['unitEnergy'] = "KCal";
	    $workout['date'] = date("Ymd", $w['dateStartTime']);
	    $workout['startTime'] = date("H:i:s.0", $w['dateStartTime']);
	    $workout['energy'] = $w['Calories'];
	    $workout['distance'] = round($w['DistanceMeters']/1000, 1);
	    $workout['speedMax'] = $w['MaximumSpeed'];
	    $workout['speedAvg'] = $w['AverageSpeed'];
	    $workout['altitudeMin'] = $w['altitudeMin'];
	    $workout['altitudeAvg'] = $w['altitudeAvg'];
	    $workout['altitudeMax'] = $w['altitudeMax'];
	    $workout['ascent'] = $w['ascent'];
	    $workout['descent'] = $w['descent'];
	    $workout['sampleHr'] = $w['sampleHr'];
	    $workout['sampleAltitude'] = $w['sampleAltitude'];
	    $workout['sampleTime'] = $w['sampleTime'];
	    $workout['sampleDistance'] = $w['sampleDistance'];
	    $workout['sampleSpeed'] = $w['sampleSpeed'];
	    $workout['sampleLocationLat'] = $w['sampleLocationLat'];
	    $workout['sampleLocationLong'] = $w['sampleLocationLong'];
	    $workout['numberOfSamples'] = $w['numberOfSamples'];

	    $workouts[] = $workout;
	}

	/*echo "<pre>\n";
	print_r($workouts);
	echo "</pre>\n";*/
	return($workouts);		
    } // end function parseHst


} // end class garmin

?>
