<?php
/**
* @version 00.04.03
* @package polaring
* @license This component in released under the GNU/GPL License
*
* File: functions/hrmParser.php
* Copied from `polarphp', found at:
* http://users.pandora.be/oppiekes/polar
*
/*

Name:				hrmParser.php
Edited:			27/11/2004
Description:	Reads and parses a workout file (hrm)

*/

function getHrmComplete($hrmFile) {
	
	/* read file */
	
	$fContents = openHrmFile($hrmFile);

	/* params block */
	
	getHrmParamsBlock($pBlock,$fContents);		

	/* hrm file version */
	
	getHrmVersion($workout,$pBlock);
	
	/* type of monitor */
	
	getHrmTypeOfMonitor($workout,$pBlock);
	
	/* recording modes */
	
	getHrmRecordingModes($workout,$pBlock);
	
	/* units full */

	specifyUnits($workout,$fContents);
	
	/* date of workout */

	getHrmDateOfWorkout($workout,$pBlock);
	
	/* workout duration */
	
	getHrmWorkoutDuration($workout,$pBlock);
	
	/* interval */
		
	getHrmInterval($workout,$pBlock);
	
	/* other param values */
	
	getHrmParamValues($workout,$pBlock);
	
	/* notes */
	
	getHrmNotes($workout,$fContents);
	
	/* lap times */
	
	getHrmLapData($workout,$fContents);	
	
	/* lap time notes */	
	
	getHrmLapDataNotes($workout,$fContents);
	
	/* limits */
	
	getHrmHrLimits($workout,$fContents);	

	/* threshold summary */
	
	getHrmTripBlock($tBlock,$fContents);
	
	getHrmDistance($workout,$tBlock);
	
	getHrmSpeed($workout,$tBlock);
	
	getHrmAltitude($workout,$tBlock);
	
	getHrmCumulativeCounters($workout,$tBlock);
		
	getHrmSampleData($workout,$fContents);

	calculateHr($workout);
	
	calculateDistanceSamples($workout);
	
	calculateCadence($workout);
	
	calculateMinAltitude($workout);
	
	return $workout;	
}
	
function openHrmFile($file) {

	if (file_exists ($file)) {
		$fContents = file($file);
	}
	else {
		dieError ("Cannot open file: $file");
	}
	
	return $fContents;

}

function getHrmParamsBlock(&$pBlock,$fContents) {

	$pBlock = getHrmBlockLines ($fContents, "Params");
	
	if (sizeof($pBlock)== 0) {
		dieError("Failed to read HRM file, can't find block 'Params' ...");
	}

}

function getHrmVersion(&$workout,$pBlock) {

	$workout['version'] = getHrmValueFromBlock($pBlock,"Version");

}

function getHrmTypeOfMonitor (&$workout,$pBlock) {

	/*Heart rate monitor type
	1 = Polar Sport Tester/ Vantage XL 2 = Polar Vantage NV	3 = Polar Accurex Plus 4 = Polar Xtrainer Plus
	7 = Polar Coach	8 = Polar S210 9 = Polar S410 9 = Polar S410 10 = Polar S510 / S520 
	11 = Polar S610 / S610i 12 = Polar S710 / S720 / S720i 13 = Polar S810 / S810i 15 = Polar E600
	20 = Polar AXN500 21 = Polar AXN700 22 = Polar S625X 23 = Polar S725
	*/
	$workout['monitor'] = getHrmValueFromBlock($pBlock,"Monitor");

}

function getHrmRecordingModes (&$workout,$pBlock) {

	if ($workout['version'] <= 105) {
		$sMode = getHrmValueFromBlock($pBlock,"Mode");
		if (strlen($sMode) != 3) {
			dieError("Mode not valid");
		}
	} elseif ($workout['version'] == 106) {
		$sMode = getHrmValueFromBlock($pBlock,"SMode");
		if ((strlen($sMode) != 9) && (strlen($sMode) != 8)) {
			dieError("SMode not valid");
		}
	}
	elseif ($workout['version'] == 107) {
		$sMode = getHrmValueFromBlock($pBlock,"SMode");
		if (strlen($sMode) != 9) {
			dieError("SMode not valid");
		}
	}
	
	if ($workout['version'] <= 105) {
		if ($sMode[0] == 0) { //0 = Cad, 1 = Alt, 3 = none
			$workout['recordingCadence'] = 1;
		} elseif ($sMode[0] == 1) {
			$workout['recordingAltitude'] = 1;
		}
		if ($sMode[1] == 1) {//0 = HR data only, 1 = HR + Cycling Data
			$workout['recordingCCData'] = 1;
		}
		if ($sMode[2] == 1) {//0 = Euro, 1 = US
			$workout['recordingUnits'] = 1;
		}
	}
	if ($workout['version'] == 106 || $workout['version'] == 107) {
		if ($sMode[0] == 1) {
			$workout['recordingSpeed'] = 1;
		} 
		if ($sMode[1] == 1) {
			$workout['recordingCadence'] = 1;
		}
		if ($sMode[2] == 1) {
			$workout['recordingAltitude'] = 1;
		}
		if ($sMode[3] == 1) {
			$workout['recordingPower'] = 1;
		} 
		if ($sMode[4] == 1) {
			$workout['recordingPowerLrb'] = 1;
		}
		if ($sMode[5] == 1) {
			$workout['recordingPowerPi'] = 1;
		}
		if ($sMode[6] == 1) {
			$workout['recordingCCData'] = 1;
		}
		if ($sMode[7] == 1) {
			$workout['recordingUnits'] = 1;
		}
		if (strlen($sMode) == 9) {
			if ($sMode[8] == 1) {
				$workout['recordingAirPressure'] = 1;
			}
		}
	}

}

function getHrmDateOfWorkout(&$workout,$pBlock) {

	$workout['date'] = getHrmValueFromBlock($pBlock, "Date");
	$workout['startTime'] = getHrmValueFromBlock($pBlock, "StartTime");
	$workout['dateStartTime'] = getHrmDateStartTime($workout["date"],$workout["startTime"]);

}

function getHrmWorkoutDuration(&$workout,$pBlock) {

	$workout['duration'] = getHrmValueFromBlock($pBlock,"Length");
	list($hour,$minute,$second) = split(":",$workout['duration'], 3);
	$second = substr($second,-5,2);
	$seconds = ($hour * 60 * 60) + ($minute * 60) + ($second);
	$workout['duration'] = fromSecondsToFull($seconds);

}

function getHrmInterval(&$workout,$pBlock) {

	$workout['interval'] = getHrmValueFromBlock($pBlock,"Interval");

}

function getHrmParamValues(&$workout,$pBlock) {

	$workout['timer1'] = getHrmValueFromBlock($pBlock,"Timer1");
	$workout['timer2'] = getHrmValueFromBlock($pBlock,"Timer2");
	$workout['timer3'] = getHrmValueFromBlock($pBlock,"Timer3");
	$workout['activeLimit'] = getHrmValueFromBlock($pBlock,"ActiveLimit");
	$workout['personalMaxHr'] = getHrmValueFromBlock($pBlock,"MaxHR");
	$workout['personalRestHr'] = getHrmValueFromBlock($pBlock,"RestHR");
	$workout['startDelay'] = getHrmValueFromBlock($pBlock,"StartDelay");
	$workout['VO2Max'] = getHrmValueFromBlock($pBlock,"VO2Max");
	$workout['weight'] = getHrmValueFromBlock($pBlock,"Weight");

}

function getHrmNotes(&$workout) {

	$nBlock = getHrmBlockLines ($fContents, "Note");
	$workout['note'] = getHrmValueFromBlock($nBlock,"Note!");

}

function getHrmLapData(&$workout,$fContents) {

	$iBlock = getHrmBlockLines ($fContents, "IntTimes");
	if (sizeof($iBlock) == 0 || sizeof($iBlock % 5)-1 !=0)  {
		dieError("Failed to read HRM file, can't find block 'IntTimes' or block not valid");
	}
	
	/* parse all laps of exercise (each lap consists of 5 lines) */
	
	$workout['numberOfLaps'] = (sizeof($iBlock)-1) / 5;
	$lapDistanceSum = 0;
	for ($i = 0; $i < $workout['numberOfLaps']; $i++) {
	
		/* lap line 1: 5 parts */
		
		$lapLine = $iBlock[$i * 5];
		list($workout['lapTimeSplit'][$i],$workout['lapHrSplit'][$i],$workout['lapHrMin'][$i],$workout['lapHrAvg'][$i],$workout['lapHrMax'][$i]) = split("\t",$lapLine, 5);
	
		/* lap line 2: 6 parts */
		
		$lapLine = $iBlock[$i * 5] + 1;
		$lapLineSplit = split("\t",$lapLine);
		
		/* column 1: misc lap time information */
		
		/* column 2: recovery time (seconds) */
		
		$workout['lapRecoveryTime'][$i] = $lapLineSplit[1];
		
		/* column 3: recovery HR (bpm) */
		$workout['lapRecoveryHr'][$i] = $lapLineSplit[2];
		
		/* column 4: momentary speed (/128) */
		
		if ($workout['recordingSpeed'] == 1) {
			$workout['lapSplitSpeed'][$i] = $lapLineSplit[3];
		}
		
		/* column 5: momentary cadence */
		
		if ($workout['recordingCadence'] == 1) {
			$workout['lapSplitCadence'][$i] = $lapLineSplit[4];
		}
		
		/* column 6: momentary altitude */
		
		if ($workout['recordingAltitude'] == 1) {
			$workout['lapSplitAltitude'][$i] = checkAltitudeData($lapLineSplit[5],$workout['version']);
		}
			
		/* lap line 3: 5 parts - only for Xtrainer Plus */
		
		/* columns 1,2,3: values of extra data series */
		
		$lapLineSplit = split("\t",$iBlock[($i * 5) + 2]);
		
		/* column 4: ascent XTr */
		
		/* column 5: lap distance value XTr */
		
		if ($workout['monitor'] = 4 && $workout['recordingAltitude'] == 1) {
			$workout["lapAscent"][$i] = $lapLineSplit[3];
			$workout["lapDistance"][$i] = $lapLineSplit[4];
		}
				
		/* lap line 4: 6 parts */
		
		$lapLineSplit = split("\t",$iBlock[($i * 5) + 3]);
		
		/* column 1: lap type identifier (Polar Coach) */
		
		/* column 2: lap distance */
		
		if ($workout['recordingSpeed'] == 1) {
			$workout['lapDistance'][$i] = $lapLineSplit[1];
		}
		
		/* column 3: momentary power value */
		
		if ($workout['recordingPower'] == true) {
			$workout['lapSplitPower'][$i] = $laplineSplit[2];
		}
		
		/* column 4: momentary temperature value */
		
		if ($workout['recordingAltitude'] == 1) {
			$workout['lapSplitTemperature'][$i] = $lapLineSplit[3] / 10;
		}
			
		//5. lap is not used
	}

}

function getHrmLapDataNotes(&$workout,$fContents) {

	$iNBlock = getHrmBlockLines ($fContents, "IntNotes");
	$numberOfNotes = (sizeof($iNBlock)-1);
	for ($i = 0; $i <= $numberOfNotes; $i++) {
		if ($numberOfNotes > $workout['numberOfLaps']) {
			dieError("Can't parse Hrm File: too many lap time notes");
		} else{
			$lapNumber  = substr($iNBlock[$i],0,1);
			$workout['intNotesLap'][$lapNumber-1] = $lapNumber;
			$lapText = substr($iNBlock[$i],1);
			$workout['intNotesText'][$lapNumber-1] = $lapText;
		}
	}

}

function getHrmHrLimits(&$workout,$fContents) {

	$s123Block = getHrmBlockLines ($fContents,"Summary-123");
	if ((sizeof($s123Block)-1)!= 7) {
		dieError("Failed to read HRM file, can't find block 'Summary-123' or block is not valid ...");
	}
	for ($i = 0; $i < 3; $i++) {
	
		/* line 1: summary for limits */
		
		$hrLine1Split = split("\t",$s123Block[$i*2]);
		if (sizeof($hrLine1Split) != 6) {
			dieError("Failed to read HRM file, can't parse 1. line of current heartrate limits in block 'Summary-123' ...");
		}
		$workout['hrLimitTotalTime'][$i] = $hrLine1Split[0];
		$workout['hrLimitTimeAboveMax'][$i] = $hrLine1Split[1];
		$workout['hrLimitTimeWithin'][$i] = $hrLine1Split[2];
		$workout['hrLimitTimeBelow'][$i] = $hrLine1Split[3];
		$workout['hrLimitTimeBetweenBelowAndRest'][$i] = $hrLine1Split[4];
		$workout['hrLimitTimeBelowRest'][$i] = $hrLine1Split[5];
		
		/* line2: limit values */
		
		$hrLine2Split = split("\t",$s123Block[$i*2+1]);
		if (sizeof($hrLine2Split) != 4) {
			dieError("Failed to read HRM file, can't parse 2. line of current heartrate limits in block 'Summary-123' ...");
		}
		$workout['hrMaxHr'][$i] = $hrLine2Split[0];
		$workout['hrLimitUpper'][$i] = $hrLine2Split[1];
		$workout['hrLimitLower'][$i] = $hrLine2Split[2];
		$workout['hrRestHr'][$i] = $hrLine2Split[3];
		
		/* vthe heartrate limits on Polar S725i (Monitor=23) are stored in percent
		=> calculate the BPM with help of max. heartrate */
		/*
		if ($workout['monitor'] == 23) {
			$workout['hrUpperLimit'][$i] = ($workout['hrMaxHr'] * ($workout['hrUpperLimit'][$i]))/100;
			$workout['hrLowerLimit'][$i] = ($workout['hrMaxHr'] * ($workout['hrLimitLower'][$i]))/100;
		}*/
	}

}

function getHrmThLimits(&$workout,$fContents) {

	$sThBlock = getHrmBlockLines ($fContents,"Summary-TH");
	if (sizeof($sThBlock)-1 != 3) {
		dieError("Failed to read HRM file, can't find block 'Summary-TH' or block is not valid ...");
	}
	
	/* line 1: summary for threshold limits */
	
	$hrThLine1Split = split("\t",$sThBlock[0]);
	if (sizeof($hrThLine1Split) != 6) {
		dieError("Failed to read HRM file, can't parse 1. line of current heartrate limits in block 'Summary-TH' ...");
	}
	
	$workout['hrThTotalTime'][$i] = $hrThLine1Split[0];
	$workout['hrThTimeAboveMax'][$i] = $hrThLine1Split[1];
	$workout['hrThTimeWithin'][$i] = $hrThLine1Split[2];
	$workout['hrThTimeBelow'][$i] = $hrThLine1Split[3];
	$workout['hrThTimeBetweenBelowAndRest'][$i] = $hrThLine1Split[4];
	$workout['hrThTimeBelowRest'][$i] = $hrThLine1Split[5];
	
	/* line2: limit values */
	
	$hrThLine2Split = split("\t",$sThBlock[1]);
	if (sizeof($hrThLine2Split) != 4) {
		dieError("Failed to read HRM file, can't parse 2. line of current heartrate limits in block 'Summary-TH' ...");
	}
	$workout['hrThMaxHr'][$i] = $hrThLine2Split[0];
	$workout['hrThUpper'][$i] = $hrThLine2Split[1];
	$workout['hrThLower'][$i] = $hrThLine2Split[2];
	$workout['hrThRestHr'][$i] = $hrThLine2Split[3];

}

function getHrmTripBlock(&$tBlock,$fContents) {

	$tBlock = getHrmBlockLines ($fContents,"Trip");

}

function getHrmDistance(&$workout,$tBlock) {

	if (sizeof($tBlock)-1 == 8) {
		if ($workout['recordingSpeed'] == 1) {
			/* line 1: distance (/10) */
			$workout['distance'] = $tBlock[0]/10;
		}
	}

}

function getHrmSpeed(&$workout,$tBlock) {

	if (sizeof($tBlock)-1 == 8) {
		if ($workout['recordingSpeed'] == 1) {
			/* line 6: average speed (/128) */
			$workout['speedAvg'] = $tBlock[5] / 10;
			/* line 7: maximum speed (/128) */		
			$workout['speedMax'] = $tBlock[6] / 10;	
		}
	}

}

function getHrmAltitude(&$workout,$tBlock) {

	if ($workout['recordingAltitude'] == 1) {
	
			/* line 2: ascent */
			
			$workout['ascent'] = checkAltitudeData($tBlock[1],$workout['version']);
			
			/* line 4: average altitude */
			
			$workout['altitudeAvg'] = checkAltitudeData($tBlock[3],$workout['version']);
			
			/* line 5: maximum altitude */
			
			$workout['altitudeMax'] = checkAltitudeData($tBlock[4],$workout['version']);	
	}

}

function getHrmCumulativeCounters(&$workout,$tBlock) {

	/* line 3: total time in seconds */
	
	$workout['ridingTimeTotal'] = $tBlock[2];
	
	/* line 8: odometer value at the end of an exercise */
	
	$workout['odometer'] = $tBlock[7];

}

function getHrmSampleData(&$workout,$fContents) {

	$hrDataBlock = getHrmBlockLines ($fContents,"HRData");
	if (sizeof($hrDataBlock) == 0) {
		dieError("Failed to read HRM file, can't find block 'HRData' or block is empty ...");
	}
	$workout['numberOfSamples'] = sizeof($hrDataBlock);
	
	/* parse each exercise sample line */
	
	for ($i = 0; $i < $workout['numberOfSamples']; $i++) {
	
		$number = 0;
		
		/* split sample line into parts */
		
		$sampleSplit = split("\t",$hrDataBlock[$i]);
		
		/* column 1 -> hr */
		
		$workout['sampleHr'][$i] = $sampleSplit[$number];
		$number++;
		
		/* column 2 -> speed (if recorded) */
		
		if (sizeof($sampleSplit) > $number && $workout['recordingSpeed'] == 1) {
			$speed = $sampleSplit[$number];
			$workout['sampleSpeed'][$i] = $speed / 10;
			$number++;
		}
		
		/* column 3 -> cadence or altitude (if recorded) (only for version 105) */
		
		if ($workout['version'] == 105) {
			if (sizeof($sampleSplit) > $number && $workout['recordingCadence'] == 1) {
				$workout['sampleCadence'][$i] = $sampleSplit[$number];
				$number++;
			}elseif (sizeof($sampleSplit) > $number && $workout['recordingAltitude'] == 1) {
				$workout['sampleAltitude'][$i] = $sampleSplit[$number] * 10;
				$number++;
			}
		}
		
		if ($workout['version'] >= 106) {
		
			/* column 3 -> cadence (if recorded) */
			
			if (sizeof($sampleSplit) > $number && $workout['recordingCadence'] == 1) {
				$workout['sampleCadence'][$i] = $sampleSplit[$number];
				$number++;
			}
			
			/* column 4 -> altitude (if recorded) */
			
			if (sizeof($sampleSplit) > $number && $workout['recordingAltitude'] == 1) {
				$workout['sampleAltitude'][$i] = $sampleSplit[$number];
				$number++;
			}
			
			/* column 5 -> power (if recorded) */
			
			if (sizeof($sampleSplit) >> $number && $workout["recordingPower"] == 1) {
				$workout['samplePower'][$i] = $sampleSplit[$number];
				$number++;
			}
			
			/* column 6 -> power balance and pedalling index (if recorded) */
			/* todo doc say value = PI*256 + LRB -> ? */
			
			if (sizeof($sampleSplit) > $number && $workout["recordingPowerLrb"] == 1 &&
		$workout['recordingPowerPi'] == true) {
				$workout['samplePowerBPI'][$i] = $sampleSplit[$number];
				$number++;
			}
		}
		if ($workout['version'] == 107) {
		
			/* column 7 -> airpressure (if recorded) */
			
			if (sizeof($sampleSplit) > $number && $workout['recordingAirPressure'] == 1) {
				$workout['sampleAirPressure'][$i] = $sampleSplit[$number];
			}
		}
	}

}

function getHrmBlockLines ($fContents,$bName) {
	$bLine = "[" . $bName . "]";
	$bFound = false;
	// process all lines of exercise file
	for ($i = 0; $i < sizeof($fContents); $i++) {
		// is this line the start of the block ?
		if (($bFound == false) && (substr($fContents[$i], 0, strlen($bLine)) == $bLine)) {
			$bFound = true;
			continue;
		}
		// is this line the end of the block ? (exit loop then)
		if (($bFound == true) && ((sizeof($fContents[$i]) == 0) || (substr($fContents[$i],0,1) == "["))) {
			break;
		}
		// if we"re inside the block, add this line to list
		if ($bFound) {
			$lines[] = $fContents[$i];
		}
	}
	for ($i = 0; $i < sizeof($lines); $i++) {
		$arrLines[] = $lines[$i];
	}
	return $arrLines;
}
	
function getHrmValueFromBlock ($bLines, $name) {
	for ($i = 0; $i < sizeof($bLines); $i++) {
		if (substr($bLines[$i],0,strlen($name)+1) == $name."="){
			return trim(substr($bLines[$i],strlen($name)+1,strlen($bLines[$i])-strlen($name)+1));
		}
	}
	return null;
}
	
function getHrmDateStartTime($date,$startTime) {
	$year = substr($date,0,4);
	$month = substr($date,4,2);
	$day = substr($date,6,2);
	list($hour,$minute,$second) = split(":",$startTime, 3);
	$dateStartTime = mktime($hour,$minute,$second,$month,$day,$year);
	return $dateStartTime;
}

function checkAltitudeData($altitude,$version) {
	if ($version == 102) {
		$altitude = $altitude * 10;
	} elseif($version >= 105) {
		$altitude = $altitude;
	} else {
		$altitude = "Invalid";
	}
	return $altitude;
}

?>
