<?php
/**
* @version 00.04.03
* @package polaring
* @license This component in released under the GNU/GPL License
*
* File: functions/parserFunctions.php
* Role: Functions for parsing workouts (srd & hrm)
*
* File is as good as copied from `polarphp', found at:
* http://users.pandora.be/oppiekes/polar
**/

function BTH($x) {
	return bin2hex($x);
}
	
function HTD($x) {
	return hexdec($x);
}

function decodeTemperature($temperature) {
	$tempValue = $temperature & 0x7f;
	if (($temperature & 0x80) != 0x80) {
		$tempValue *= -1;
	}
	return $tempValue;
}

function decodeAltitude ($lsb, $msb) {
	$altitude = ($lsb + (($msb & 0x7f) << 8));
	if (($msb & 0x80) != 0x80) {
		$altitude *= -1;
	}
	return $altitude;
}

function decodeChar ($value) {
            
	$cDecoded = '?';
	
	switch ($value) {

		case 0: case 1: case 2: case 3: case 4: 
		case 5: case 6: case 7: case 8: case 9:
		$cDecoded = ('0' + $value); break;
                case 10:
		$cDecoded = ' '; break;
		case 11: case 12: case 13: case 14: case 15:
		case 16: case 17: case 18: case 19: case 20:
		case 21: case 22: case 23: case 24: case 25:
		case 26: case 27: case 28: case 29: case 30:
		case 31: case 32: case 33: case 34: case 35:
		case 36:
                $cDecoded = ('A' + $value - 11); break;
		case 37: case 38: case 39: case 40: case 41:
		case 42: case 43: case 44: case 45: case 46:
		case 47: case 48: case 49: case 50: case 51:
		case 52: case 53: case 54: case 55: case 56:
		case 57: case 58: case 59: case 60: case 61:
                case 62:
		$cDecoded = ('a' + $value - 37); break;
		case 63: $cDecoded = '-'; break;
		case 64: $cDecoded = '%'; break;
		case 65: $cDecoded = '/'; break;
		case 66: $cDecoded = '('; break;
		case 67: $cDecoded = ')'; break;
		case 68: $cDecoded = '*'; break;
		case 69: $cDecoded = '+'; break;
		case 70: $cDecoded = '.'; break;
		case 71: $cDecoded = ':'; break;
		case 72: $cDecoded = '?'; break;
		default: break;
	}
    
	return $cDecoded;        
        
}

function calculateHr(&$workout) {
	$avgHrSum = 0;
	$workout['hrMax'] = 0;
	for ($i = 0; $i < $workout['numberOfSamples']; $i++) {
		$avgHrSum = $avgHrSum + $workout['sampleHr'][$i];
		if ($workout['sampleHr'][$i] > $workout['hrMax']) {
			$workout['hrMax'] = $workout['sampleHr'][$i];
		}
	}
	$workout['hrAvg'] = round($avgHrSum / $workout['numberOfSamples']);

}


function trimAltitude(&$workout) {
    if ( !is_array($workout['sampleAltitude']) ) {
	return;
    }	
    if ( count($workout['sampleAltitude']) > 0 ) {
	for ( $i = 0; $i < count($workout['sampleAltitude']); $i++ ) {
	    $alt[$i] = trim($workout['sampleAltitude'][$i]);
	}
    }
    unset($workout['sampleAltitude']);
    $workout['sampleAltitude'] = $alt;
}

function calculateDistanceSamples(&$workout) {

	if ($workout['recordingSpeed'] == true) {
		$distanceSum = 0;
		for ($i = 0; $i < $workout['numberOfSamples']; $i++) {
			$workout['sampleDistance'][$i] = round($distanceSum/1000,3);
			$distanceSum += ($workout['sampleSpeed'][$i] * $workout['interval']) / 3.6;
		}
	}

}

function calculateCadence(&$workout) {

	if ($workout['recordingCadence'] == 1) {
		$cadenceAvgSum = 0;
		$workout['cadenceMax'] = 0;
		for ($i = 0; $i < $workout['numberOfSamples']; $i++) {
			$cadenceAvgSum += $workout['sampleCadence'][$i];
				if ($workout['sampleCadence'][$i] > $workout['cadenceMax']) {
					$workout['cadenceMax'] = $workout['sampleCadence'][$i];
				}
		}
		$workout['cadenceAvg'] = Round ($cadenceAvgSum / $workout['numberOfSamples']);
	}	

}

function calculateTimeSamples(&$workout) {

	$timeSum = 0;
	for ($i = 0; $i < $workout['numberOfSamples']; $i++) {
		$sampleTime = $timeSum;
		$workout['sampleTime'][$i] = fromSecondsToFull($sampleTime);
		$timeSum = $timeSum + $workout['interval'];
	}

	return $value;

}

function calculateMinAltitude($workout) {

	if ($workout['recordingAltitude'] == 1) {
		$workout['altitudeMin'] = 100000;
		for ($i = 0; $i < $workout['numberOfSamples']; $i++) {
			if ($workout['sampleAltitude'][$i] < $workout['altitudeMin']) {
				$workout['altitudeMin'] = $workout['sampleAltitude'][$i];
			}
		}
		$workout['altitudeMin'] = checkAltitudeData($workout['altitudeMin'],$workout['version']);
	}

}

function repairDistanceSamples($distance) {
}

function fromSecondsToFull($time) {
	$hours = checkLength(floor($time/3600));
	$hoursRest = $time%3600;
	$minutes = checkLength(floor($hoursRest/60));
	$seconds = checkLength($hoursRest%60);
	$sampleTime = $hours . ":" . $minutes. ":" . $seconds;
	return $sampleTime;
}

function checkLength($time) {
	if ($time < 10) {
		return "0".$time;
	} else {
		return $time;
	}
}

function totalTimeSummary($workout) {

	$totalTime = 0;
	foreach ($workout as $w) {
		list($hour,$minute,$second) = split(":",$w["duration"], 3);
		$second = substr($second,-5,2);
		$seconds = ($hour * 60 * 60) + ($minute * 60) + ($second);
		$totalTime = $totalTime + $seconds;
	}

	return fromSecondsToFull($totalTime);

}

function totalSummary($workout,$value) {

	$totalValue = 0;
	foreach ($workout as $w) {
		$totalValue = $totalValue + $w[$value];
	}

	return $totalValue;
}

function fullTime($hours,$minutes,$seconds) {

	$time = ($hours*60*60) + ($minutes * 60) + ($seconds);
	$time = $hours.":".$minutes.":".$seconds;
	return $time;

}

function workoutHour($workout) {
	
	$hour['0-1'] = 0;
	$hour['1-2'] = 0;
	$hour['2-3'] = 0;
	$hour['3-4'] = 0;
	$hour['4-5'] = 0;
	$hour['5-6'] = 0;
	$hour['6+'] = 0;
	
	foreach ($workout as $w) {
		switch (substr($w['duration'],0,2)) {
			case 00: $hour['0-1'] += 1; break;
			case 01: $hour['1-2'] += 1; break;
			case 02: $hour['2-3'] += 1; break;
			case 03: $hour['3-4'] += 1; break;
			case 04: $hour['4-5'] += 1; break;
			case 05: $hour['5-6'] += 1; break;
			default: $hour['6+'] += 1; break;
		}
	}
	
	return $hour;	

}

?>
